// SPDX-License-Identifier: GPL-2.0-only OR GPL-3.0-only OR LicenseRef-KDE-Accepted-GPL
// SPDX-FileCopyrightText: 2021 Harald Sitter <sitter@kde.org>

#pragma once

#include "smartctl.h"

class SimulationCtl : public AbstractSMARTCtl
{
    Q_OBJECT
public:
    void run(const QString &devicePath) override;
};
